import numpy as np 

def c2f(x):
	""" Converts 'x' to a float """
	try:
		x = int(x)
		return x
	except ValueError:
		try:
			x = float(x)
			return x
		except ValueError:
			return x

def grad_magnitude(gx, gy, gz):
	"""Converts the separate gradient magnitudes to a single magnitude
Args:
	gx/y/z : fields in x y and z directions 2D array
Returns:
	grad_mag : gradient of fields at each point"""
	grad_mag = gx
	for i in range(gx.shape[0]):
		for j in range(gy.shape[1]):
			for k in range(gz.shape[2]):
				grad_mag[i,j,k] = np.sqrt(gx[i,j,k]**2+gy[i,j,k]**2+gz[i,j,k]**2)

	return grad_mag


from . import math_functions as mf 
from ase.visualize import view
import pandas as pd
import numpy as np
import ase.io.cube
import pickle
import os

def read_cube_file(filename):

	potential, atoms = ase.io.cube.read_cube_data(filename)
	vector_a = np.linalg.norm(atoms.cell[1])
	vector_b = np.linalg.norm(atoms.cell[1])
	vector_c = np.linalg.norm(atoms.cell[2])
	NGX = len(potential)
	NGY = len(potential[0])
	NGZ = len(potential[0][0])
	resolution_x = vector_a/NGX
	resolution_y = vector_b/NGY
	resolution_z = vector_c/NGZ
	# print(type(atoms))
	coords = atoms.get_positions()
	atom_types = atoms.get_chemical_symbols()

	with open(filename, 'r') as cube:
		lines = cube.readlines()
	origin = [float(i) for i in lines[2].split()[1:-1]]

	# symbols = atoms.symbols

	return potential, NGX, NGY, NGZ, atoms, [vector_a, vector_b, vector_c], coords, atom_types, origin, [resolution_x, resolution_y, resolution_z]

def load_obj(name):
#	with open(name, 'rb') as f:
#		return pickle.load(f)
	return pickle.load(open(name, 'rb'))
	
def get_te(species, num_atoms):
	""" Returns the total number of valence electrons for the unit cell """
	tot_electrons = {'Ru': 8.0, 'Re': 7.0, 'Rf': 261.0, 'Ra': 226.0, 'Rb': 85.47, 'Rn': 8.0, 
					'Rh': 9.0, 'Be': 2.0, 'Ba': 137.33, 'Bh': 264.0, 'Bi': 5.0, 'Bk': 247.0, 
					'Br': 7.0, 'H': 1.0, 'P': 5.0, 'Os': 8.0, 'Es': 252.0, 'Hg': 12.0, 'Ge': 4.0, 
					'Gd': 18.0, 'Ga': 3.0, 'Pr': 13.0, 'Pt': 10.0, 'Pu': 16.0, 'C': 4.0, 'Pb': 4.0, 
					'Pa': 13.0, 'Pd': 10.0, 'Cd': 12.0, 'Po': 6.0, 'Pm': 15.0, 'Hs': 269.0, 'Ho': 21.0, 
					'Hf': 4.0, 'K': 39.1, 'He': 2.0, 'Md': 258.0, 'Mg': 2.0, 'Mo': 6.0, 'Mn': 7.0, 'O': 6.0, 
					'Mt': 268.0, 'S': 6.0, 'W': 6.0, 'Zn': 12.0, 'Eu': 17.0, 'Zr': 91.22, 'Er': 22.0, 
					'Ni': 10.0, 'No': 259.0, 'Na': 1.0, 'Nb': 92.91, 'Nd': 14.0, 'Ne': 8.0, 'Np': 15.0, 
					'Fr': 223.0, 'Fe': 8.0, 'Fm': 257.0, 'B': 3.0, 'F': 7.0, 'Sr': 87.62, 'N': 5.0, 
					'Kr': 8.0, 'Si': 4.0, 'Sn': 4.0, 'Sm': 16.0, 'V': 5.0, 'Sc': 3.0, 'Sb': 5.0, 
					'Sg': 266.0, 'Se': 6.0, 'Co': 9.0, 'Cm': 18.0, 'Cl': 7.0, 'Ca': 40.08, 'Cf': 251.0, 
					'Ce': 12.0, 'Xe': 8.0, 'Lu': 25.0, 'Cs': 132.91, 'Cr': 6.0, 'Cu': 11.0, 'La': 11.0, 
					'Li': 1.0, 'Tl': 3.0, 'Tm': 23.0, 'Lr': 262.0, 'Th': 12.0, 'Ti': 4.0, 'Te': 6.0, 
					'Tb': 19.0, 'Tc': 7.0, 'Ta': 5.0, 'Yb': 24.0, 'Db': 262.0, 'Dy': 20.0, 'I': 7.0, 
					'U': 14.0, 'Y': 88.91, 'Ac': 11.0, 'Ag': 11.0, 'Ir': 9.0, 'Am': 17.0, 'Al': 3.0, 
					'As': 5.0, 'Ar': 8.0, 'Au': 11.0, 'At': 7.0, 'In': 3.0}

	tot_e = 0
	for line, item in enumerate(species):
		tot_e += tot_electrons[str(item)]*num_atoms[line]
	return tot_e

def get_n(species, num_atoms):
	""" Returns 'n' the scaling factor for the PV and SA calcs """
	atomic_mass = {'H':1.01, 'He':4.00, 'Li':6.94, 'Be':9.01, 'B':10.81, 'C':12.01,
					'N':14.01, 'O':16.00, 'F':19.00, 'Ne':20.18, 'Na':22.99, 'Mg':24.31,
					'Al':26.98, 'Si':28.09, 'P':30.97, 'S':32.07, 'Cl':35.45, 'Ar':39.95,
					'K':39.10, 'Ca':40.08, 'Sc':44.96, 'Ti':47.87, 'V':50.94, 'Cr':52.00,
					'Mn':54.94, 'Fe':55.85, 'Co':58.93, 'Ni':58.69, 'Cu':63.55, 'Zn':65.39,
					'Ga':69.72, 'Ge':72.61, 'As':74.92, 'Se':78.96, 'Br':79.90, 'Kr':83.80,
					'Rb':85.47, 'Sr':87.62, 'Y':88.91, 'Zr':91.22, 'Nb':92.91, 'Mo':95.94,
					'Tc':98.00, 'Ru':101.07, 'Rh':102.91, 'Pd':106.42, 'Ag':107.87,
					'Cd':112.41, 'In':114.82, 'Sn':118.71, 'Sb':121.76, 'Te':127.60,
					'I':126.90, 'Xe':131.29, 'Cs':132.91, 'Ba':137.33, 'La':138.91,
					'Ce':140.12, 'Pr':140.91, 'Nd':144.24, 'Pm':145.00, 'Sm':150.36,
					'Eu':151.96, 'Gd':157.25, 'Tb':158.93, 'Dy':162.50, 'Ho':164.93,
					'Er':167.26, 'Tm':168.93, 'Yb':173.04, 'Lu':174.97, 'Hf':178.49,
					'Ta':180.95, 'W':183.84, 'Re':186.21, 'Os':190.23, 'Ir':192.22,
					'Pt':195.08, 'Au':196.97, 'Hg':200.59, 'Tl':204.38, 'Pb':207.2,
					'Bi':208.98, 'Po':209.00, 'At':210.00, 'Rn':222.00, 'Fr':223.00,
					'Ra':226.00, 'Ac':227.00, 'Th':232.04, 'Pa':231.04, 'U':238.03,
					'Np':237.00, 'Pu':244.00, 'Am':243.00, 'Cm':247.00, 'Bk':247.00,
					'Cf':251.00, 'Es':252.00, 'Fm':257.00, 'Md':258.00, 'No':259.00,
					'Lr':262.00, 'Rf':261.00, 'Db':262.00, 'Sg':266.00, 'Bh':264.00,
					'Hs':269.00, 'Mt':268.00}
	if num_atoms == 'None':
		molar_mass = 0
		for line, item in enumerate(species):
			molar_mass += atomic_mass[str(item)]
		return 1/molar_mass
	else:
		molar_mass = 0
		for line, item in enumerate(species):
			molar_mass += atomic_mass[str(item)]*num_atoms[line]
		return 1/molar_mass

def unit_cell_volume(lattice):
	""" returns the unit cell volume and vector properties """
	def angle(v1, v2):
		angle = np.arccos((np.dot(v1, v2))/(np.linalg.norm(v1)*np.linalg.norm(v2)))
		return angle

	vec = [np.sqrt(lattice[0,0]**2+lattice[0,1]**2+lattice[0,2]**2), 
		np.sqrt(lattice[1,0]**2+lattice[1,1]**2+lattice[1,2]**2),
		np.sqrt(lattice[2,0]**2+lattice[2,1]**2+lattice[2,2]**2)]

	cb = angle(lattice[2], lattice[1])
	ca = angle(lattice[2], lattice[0])
	ab = angle(lattice[0], lattice[1])

	vp1 = vec[0]*vec[1]*vec[2]
	vp2 = np.sqrt((1-(np.cos(cb))**2 - (np.cos(ca))**2 - (np.cos(ab))**2) + 2*np.cos(ca)*np.cos(cb)*np.cos(ab))

	vol = vp1 * vp2

	return vol, vec

def process_LOCPOT(file):

	def pot_grid(ng, vasp_pot, lattice, file):
		""" Generate potential grid for all calcs """
		l = 0
		potential_grid = np.zeros(shape=(ng[0], ng[1], ng[2]))
		if file == 'LOCPOT':
			for k in range(ng[2]):
				for j in range(ng[1]):
					for i in range(ng[0]):
						potential_grid[i, j, k] = vasp_pot[l]
						l += 1

		else:
			a = np.sqrt(lattice[0,0]**2+lattice[0,1]**2+lattice[0,2]**2)
			b = np.sqrt(lattice[1,0]**2+lattice[1,1]**2+lattice[1,2]**2)
			c = np.sqrt(lattice[2,0]**2+lattice[2,1]**2+lattice[2,2]**2)

			volume = (a/ng[0]) * (b/ng[1]) * (c/ng[2]) * np.prod(ng)

			for k in range(ng[2]):
				for j in range(ng[1]):
					for i in range(ng[0]):
						potential_grid[i,j,k] = vasp_pot[l]/volume
						l += 1

		return potential_grid

	# read in the file
	with open(file, 'r') as f:
		lines = f.read().split('\n')

	# get the lattice
	lattice = np.zeros(shape=(3,3))
	for i in range(2, 5):
		line = lines[i].split()
		lattice[i-2][0] = float(line[0])
		lattice[i-2][1] = float(line[1])
		lattice[i-2][2] = float(line[2])

	# get the unit cell volume and vectors
	ucv, vec = unit_cell_volume(lattice)

	# get the species and number of each
	species = lines[5].split()
	species[0] = species[0].lstrip().rstrip()
	species = list(filter(None, species))

	num_species = len(species)
	atomcount = lines[6].split()
	num_atoms = []
	total_num_atoms = 0
	for i in atomcount:
		total_num_atoms += mf.c2f(i)
		num_atoms.append(mf.c2f(i))
	print(species)

	# get 'n' the scaling factor for PV and SA calcs
	n = get_n(species, num_atoms)
	print(n)
	# calc crystal density
	cryst_density = (1/n)/(ucv*6.022e23*1e-24)

	# get total number of electrons
	total_num_electrons = get_te(species, num_atoms)

	# get the coordinates for all of the atoms
	coords = np.zeros(shape=(total_num_atoms, 3))
	for i in range(8, total_num_atoms + 8):
		line = lines[i].split()
		coords[i-8, 0] = mf.c2f(line[0])
		coords[i-8, 1] = mf.c2f(line[1])
		coords[i-8, 2] = mf.c2f(line[2])

	# extract NGX, NGY, NGZ as a list
	line = lines[total_num_atoms + 9].split()
	ng = [int(i) for i in line]

	# extract all of the potential information!!!
	k = 0
	bottom = total_num_atoms + 10
	top = bottom + int(np.prod(ng)/5)
	vasp_pot = np.zeros(shape=(int(np.prod(ng))))
	for i in range(bottom, top):
		line = lines[i].split()

		vasp_pot[k] = line[0]
		vasp_pot[k+1] = line[1]
		vasp_pot[k+2] = line[2]
		vasp_pot[k+3] = line[3]
		vasp_pot[k+4] = line[4]

		k += 5

	pg = pot_grid(ng, vasp_pot, lattice, file)

	# return all of the necessary properties
	return lattice, coords, vasp_pot, ng, n, total_num_atoms, total_num_electrons, species, num_atoms, ucv, vec, cryst_density, pg

def process_POSCAR(file):
	with open(file, 'r') as f:
		lines = f.read().split('\n')

	# get the lattice
	lattice = np.zeros(shape=(3,3))
	for i in range(2, 5):
		line = lines[i].split()
		lattice[i-2][0] = float(line[0])
		lattice[i-2][1] = float(line[1])
		lattice[i-2][2] = float(line[2])

	# get the unit cell volume and vectors
	ucv, vec = unit_cell_volume(lattice)

	# get the species and number of each
	species = lines[5].split()
	species[0] = species[0].lstrip().rstrip()
	species = filter(None, species)

	num_species = len(species)
	atomcount = lines[6].split()
	num_atoms = []
	total_num_atoms = 0
	for i in atomcount:
		total_num_atoms += mf.c2f(i)
		num_atoms.append(mf.c2f(i))
	print(species)

	# get 'n' the scaling factor for PV and SA calcs
	n = get_n(species, num_atoms)

	# calc crystal density
	cryst_density = (1/n)/(ucv*6.022e23*1e-24)

	# get total number of electrons
	total_num_electrons = get_te(species, num_atoms)

	# get the coordinates for all of the atoms
	coords = np.zeros(shape=(total_num_atoms, 3))
	for i in range(9, total_num_atoms + 8):
		line = lines[i].split()
		coords[i-9, 0] = mf.c2f(line[0])
		coords[i-9, 1] = mf.c2f(line[1])
		coords[i-9, 2] = mf.c2f(line[2])

	return lattice, coords, n, total_num_atoms, total_num_electrons, species, num_atoms, ucv, vec, cryst_density

def process_CHGCAR(file):

	def pot_grid(ng, vasp_pot, lattice, file):
		""" Generate potential grid for all calcs """
		l = 0
		potential_grid = np.zeros(shape=(ng[0], ng[1], ng[2]))
		if file == 'LOCPOT':
			for k in range(ng[2]):
				for j in range(ng[1]):
					for i in range(ng[0]):
						potential_grid[i, j, k] = vasp_pot[l]
						l += 1

		else:
			a = np.sqrt(lattice[0,0]**2+lattice[0,1]**2+lattice[0,2]**2)
			b = np.sqrt(lattice[1,0]**2+lattice[1,1]**2+lattice[1,2]**2)
			c = np.sqrt(lattice[2,0]**2+lattice[2,1]**2+lattice[2,2]**2)

			volume = (a/ng[0]) * (b/ng[1]) * (c/ng[2]) * np.prod(ng)

			for k in range(ng[2]):
				for j in range(ng[1]):
					for i in range(ng[0]):
						potential_grid[i,j,k] = vasp_pot[l]/volume
						l += 1
			print('GOT INTO THE ELSE STATEMENT')

		return potential_grid

	# read in the file
	with open(file, 'r') as f:
		lines = f.read().split('\n')

	# get the lattice
	lattice = np.zeros(shape=(3,3))
	for i in range(2, 5):
		line = lines[i].split()
		lattice[i-2][0] = float(line[0])
		lattice[i-2][1] = float(line[1])
		lattice[i-2][2] = float(line[2])

	# get the unit cell volume and vectors
	ucv, vec = unit_cell_volume(lattice)

	# get the species and number of each
	species = lines[5].split()
	species[0] = species[0].lstrip().rstrip()
	species = list(filter(None, species))

	num_species = len(species)
	atomcount = lines[6].split()
	num_atoms = []
	total_num_atoms = 0
	for i in atomcount:
		total_num_atoms += mf.c2f(i)
		num_atoms.append(mf.c2f(i))
	print(species)

	# get 'n' the scaling factor for PV and SA calcs
	n = get_n(species, num_atoms)
	print(n)
	# calc crystal density
	cryst_density = (1/n)/(ucv*6.022e23*1e-24)

	# get total number of electrons
	total_num_electrons = get_te(species, num_atoms)

	# get the coordinates for all of the atoms
	coords = np.zeros(shape=(total_num_atoms, 3))
	for i in range(8, total_num_atoms + 8):
		line = lines[i].split()
		coords[i-8, 0] = mf.c2f(line[0])
		coords[i-8, 1] = mf.c2f(line[1])
		coords[i-8, 2] = mf.c2f(line[2])

	# extract NGX, NGY, NGZ as a list
	line = lines[total_num_atoms + 9].split()
	ng = [int(i) for i in line]

	# extract all of the potential information!!!
	k = 0
	bottom = total_num_atoms + 10
	top = bottom + int(np.prod(ng)/5)
	vasp_pot = np.zeros(shape=(int(np.prod(ng))))
	for i in range(bottom, top):
		line = lines[i].split()

		vasp_pot[k] = line[0]
		vasp_pot[k+1] = line[1]
		vasp_pot[k+2] = line[2]
		vasp_pot[k+3] = line[3]
		vasp_pot[k+4] = line[4]

		k += 5

	pg = pot_grid(ng, vasp_pot, lattice, file)

	# return all of the necessary properties
	return lattice, coords, vasp_pot, ng, n, total_num_atoms, total_num_electrons, species, num_atoms, ucv, vec, cryst_density, pg



from . import geometry_manipulation_functions as gm
from . import process_functions as pf 
from . import math_functions as mf 
from tqdm import tqdm
import pandas as pd 
import numpy as np 
import pickle
import shutil
import math
import os


class Material:
	""" Class for properties of material """
	def __init__(self, file):
		lattice, coords, vasp_pot, ng, n, total_num_atoms, total_num_electrons, species, num_atoms, ucv, vec, cryst_density, pg = pf.process_LOCPOT(file)

		# assign material properties
		self.type = file
		self.lattice = lattice
		self.coords = coords
		self.ngs = ng
		self.vasp_pot = vasp_pot
		self.n = n
		self.species = species
		self.num_atoms = num_atoms
		self.total_num_atoms = total_num_atoms
		self.ucv = ucv
		self.vecs = vec 
		self.cryst_density = cryst_density
		self.pot_grid = pg
		self.coords_and_atoms = gm.combine_coordsANDatoms(coords, species, num_atoms=num_atoms)

	def save_obj(self, name_path):
		""" save material as a dictionary output """
		with open(name_path, 'wb') as f:
			pickle.dump(self, f, pickle.HIGHEST_PROTOCOL)

	def get_efield(self):
		n = self.n
		ng = self.ngs
		vecs = self.vecs

		pgrid = self.pot_grid
		res = [vecs[0]/ng[0], vecs[1]/ng[1], vecs[2]/ng[2]]

		grad_x, grad_y, grad_z = np.gradient(pgrid[:,:,:], res[0], res[1], res[2])

		grad_mag = mf.grad_magnitude(grad_x, grad_y, grad_z)

		self.efield = grad_mag

		# oefield, oefield_var = ef.get_field(grad_mag, ng, res)
		# self.oefield = oefield

	def sample_field(self):
		cv = 1e-5
		# cv = 1e-6
		efield = self.efield
		ngs = self.ngs
		vecs = self.vecs

		vacuum = []
		non_vacuum = []
		surface = []
		vac_ijk = []
		nvac_ijk = []
		surf_ijk = []

		v = 0
		n = 0
		sarea = 0
		mi = (vecs[1]/ngs[1]) * (vecs[2]/ngs[2])
		mj = (vecs[0]/ngs[0]) * (vecs[2]/ngs[2])
		mk = (vecs[0]/ngs[0]) * (vecs[1]/ngs[1])
		print(mi, mj, mk)
		for i in tqdm(range(ngs[0] - 1)):
			for j in range(ngs[1] - 1):
				for k in range(ngs[2] - 1):
					if np.absolute(efield[i,j,k] - efield[i+1, j+1, k+1]) <= cv:
						# if efield[i,j,k] - efield[i + 1, j + 1, k + 1] <= cv:
						vacuum.append(efield[i,j,k])
						v += 1
					else:
						non_vacuum.append(efield[i,j,k])
						nvac_ijk.append([i, j, k])
						n += 1
					if (n+v) == 1:
						surface.append(efield[i,j,k])
						surf_ijk.append([i, j, k])
						sarea += mk
					elif v == 2 or n == 2:
						v = 0
						n = 0
		"""
		for i in tqdm(range(ngs[0]-2, ngs[0]-1)):
			for j in range(ngs[1]-2, ngs[1]-1):
				for k in range(ngs[2]-2, ngs[2]-1):
					if np.absolute(efield[i,j,k]-efield[i+1,j+1,k+1]) <=cv:
						vacuum.append(efield[i,j,k])
						v+=1
					else:
						non_vacuum.append(efield[i,j,k])
						nvac_ijk.append([i,j,k])
						n+=1
					if (n+v) == 1:
						surface.append(efield[i,j,k])
						surf_ijk.append([i,j,k])
					elif v == 2 or n == 2:
						v = 0
						n = 0
		v = 0
		n = 0
		for j in tqdm(range(ngs[1]-1)):
			for k in range(ngs[2]-1):
				for i in range(ngs[0]-1):
					if np.absolute(efield[i,j,k] - efield[i+1,j+1,k+1]) <= cv:
						#vacuum.append(efield[i,j,k])
						v+=1
					else:
						#non_vacuum.append(efield[i,j,k])
						#nvac_ijk.append([i,j,k])
						n+=1
					# commenting out .appends in this one, because don't want duplicate pts
					if (n+v) == 1:
						#surface.append(efield[i,j,k])
						sarea += mi
					elif v==2 or n==2:
						v = 0
						n = 0
		v = 0
		n = 0
		for k in tqdm(range(ngs[2]-1)):
			for i in range(ngs[0]-1):
				for j in range(ngs[1]-1):
					if np.absolute(efield[i,j,k]-efield[i+1,j+1,k+1]) <= cv:
						v+=1
					else:
						n += 1
					if (n+v) == 1:
						sarea += mj
					elif v == 2 or n == 2:
						v = 0
						n = 0
		"""
		self.vac = vacuum
		self.nvac = non_vacuum
		self.surf = surface
		self.sijk = surf_ijk
		self.nijk = nvac_ijk
		self.surf_area = sarea * float(1e-20)*float(self.n)*float(6.022e23)
		

	def calc_volume(self, vol_type):
		ng = self.ngs
		vecs = self.vecs
		vol_per_cube = (vecs[0]/ng[0]) * (vecs[1]/ng[1]) * (vecs[2]/ng[2])

		if vol_type == 'MOF_pore':
			vol = float(vol_per_cube)*len(self.vac)*float(1e-24)*float(self.n)*float(6.022e23)
		elif vol_type == 'MOF_pore_orig':
			vol = float(vol_per_cube)*len(self.vac)*float(1e-24)*float(self.n)*float(6.022e23) * 8
		elif vol_type == 'molecule':
			vol = float(vol_per_cube) * len(self.nvac)

		self.vol = vol

	def calc_surfacearea(self):
		ng = self.ngs
		vecs = self.vecs
		sijk = np.array(self.sijk[::10])
		print(sijk.shape)
		nvac = np.array(self.nijk[::10])
		print(nvac.shape)
		# vx = [nvac[r][0]*ng[0] for r in range(vr)]
		# vy = [nvac[r][1]*ng[1] for r in range(vr)]
		# vz = [nvac[r][2]*ng[2] for r in range(vr)]

		# try:
		# cvf.create_vacs(vx, vy, vz)
		# except Exception:
		# cvf.remove_vacs(vx)
		path = '/home/hmspanama/Desktop/CMOFs/label_CoREMOFs'
		try:
			os.makedirs(path + '/sepvac/')		
		except OSError:
			shutil.rmtree(path + '/sepvac/')
			os.makedirs(path + '/sepvac/')
		ca = np.zeros(shape=(6,4))
		surface_area = 0

		mi = (vecs[1]/ng[1]) * (vecs[2]/ng[2])
		mj = (vecs[0]/ng[0]) * (vecs[2]/ng[2])
		mk = (vecs[0]/ng[0]) * (vecs[1]/ng[1])
		
		# avgms = (mi+mj+mk)/3
		avgms = (mi*mj + mi*mk + mk*mj)
		avgsurf = float(avgms) * len(self.surf) * float(1e-20)*float(self.n)*float(6.022e23)
		print(avgms)
		print(avgsurf)

		hik = np.sqrt(mi**2 + mk**2)
		hij = np.sqrt(mi**2 + mj**2)
		hkj = np.sqrt(mk**2 + mj**2)
		p = (hik + hij + hkj)/3
		planearea = np.sqrt(p*(p-hik)*(p-hij)*(p-hkj))
		planesurf = float(planearea * avgms)*len(self.surf)*float(1e-20)*float(self.n)*float(6.022e23)
		print(planesurf)
		print(ng[0]*ng[1]*ng[2])
		print(len(self.vac) + len(self.nvac))
		print(len(self.surf))
		"""
		print(nvac[:].shape)
		cvf.create_vacs(nvac[:,0], nvac[:,1], nvac[:,2])
				
		tic = time.time()
		print('pre sa loop >>')
		x = sijk[:,0]
		y = sijk[:,1]
		z = sijk[:,2]
		s = 0
		print(len(x), len(y), len(z))
		for ii in tqdm(range(len(x) - 1)):
			ca[0] = [x[ii]+1, y[ii], z[ii], mi]
			ca[1] = [x[ii]-1, y[ii], z[ii], mi]
			ca[2] = [x[ii], y[ii]+1, z[ii], mj]
			ca[3] = [x[ii], y[ii]-1, z[ii], mj]
			ca[4] = [x[ii], y[ii], z[ii]+1, mk]
			ca[5] = [x[ii], y[ii], z[ii]-1, mk]
			print(s)
			s += float(cvf.mpi_calc(ca))
		"""
		sarea_total = float(s) * float(self.n) * float(1e-20) * float(6.022e23)

		print(sarea_total)
		self.surf_area = sarea_total


	def original_sampling_method(self):
		""" Samples electric field generated by get_efield(self) """
		""" Returns vacuum, non_vacuum, surface, and surface i,j,k values """
		cv = 1e-5 # sigfigs

		ng = self.ngs
		potential_grid = self.efield
		vecs = self.vecs

		cube = [2,2,2]
		travelled = [0,0,0]
		origin = [0,0,0]
		# try:
		vacuum = []
		non_vacuum = []
		surface = []
		
		vacpot = []
		nvacpot = []
		surfpot = []
		surf_ijk = []
		cubepot = []
		
		n = 0
		v = 0

		for i in tqdm(range(0, ng[0], cube[0])):
			for j in range(0, ng[1], cube[1]):
				for k in range(0, ng[2], cube[2]):
					origin = [float(i)/ng[0], float(j)/ng[1], float(k)/ng[2]]
					n_origin = np.zeros(shape = (3))
					n_origin[0] = int(i)
					n_origin[1] = int(j)
					n_origin[2] = int(k)

					density_cube = np.zeros(shape=(cube[0], cube[1], cube[2]))

					for x in range(0, cube[0]):
						for y in range(0, cube[1]):
							for z in range(0, cube[2]):
								xv = n_origin[0]+travelled[0]+x
								yv = n_origin[1]+travelled[1]+y
								zv = n_origin[2]+travelled[2]+z

								zv = zv - ng[2]*round(zv/ng[2])
								yv = yv - ng[1]*round(yv/ng[1])
								xv = xv - ng[0]*round(xv/ng[0])

								density_cube[x, y, z] = potential_grid[int(xv), int(yv), int(zv)]
					cube_potential = density_cube.mean()
					cubepot.append(cube_potential)
					cube_var = np.var(density_cube)
					if cube_var <= cv:
						vacuum.append(origin)
						vacpot.append(cube_potential)
						v += 1
					else:
						non_vacuum.append(origin)
						nvacpot.append(cube_potential)
						n += 1
					if (n+v) == 2:
						surface.append(origin)
						surfpot.append(cube_potential)
						surf_ijk.append([[i, j, k]])
					elif v == 2 or n == 2:
						v = 0
						n = 0
		
		# vol_per_cube = (vecs[0]/ng[0]) * (vecs[1]/ng[1]) * (vecs[2]/ng[2])

		# mof_vol = 8*float(vol_per_cube)*len(vacuum)*float(1e-24)*float(self.n)*float(6.022e23)
		# ubound_vol = vol_per_cube * len(non_vacuum) * 8
		self.vac = vacuum
		self.nvac = non_vacuum
		self.surf = surface
		self.sijk = surf_ijk
		# self.ovol = ubound_vol



from . import geometry_manipulation_functions as gm
from . import process_functions as pf 
from . import math_functions as mf 
from tqdm import tqdm
import pandas as pd 
import numpy as np 
import pickle
import math
import os
import ase

# -------------------------------------------------------------------------------------------------------
# Defines object type Molecule >> can handle Gaussian .cube output file
# -------------------------------------------------------------------------------------------------------

class Molecule:
    """ Class for properties of single molecules """

    def __init__(self, file):
        """ read in cube file using ase, and assign material properties """
        potential, ngx, ngy, ngz, atoms, lattice, coords, species, origin, resolution = pf.read_cube_file(file)
        self.pot_grid = potential
        self.ngs = [ngx, ngy, ngz]
        self.atoms = atoms
        self.lattice = lattice
        self.coords = coords
        self.species = species
        # get 'n' the scaling factor for PV and SA calcs
        self.n = pf.get_n(species, 'None')
        # self.atom_symbols = symbols
        self.coords_and_atoms = gm.combine_coordsANDatoms(coords, species)
        self.vecs = lattice
        self.origin = origin
        self.res = resolution
        print(atoms)

    def read_cube_file(self, filename):
        """ reads in Gaussian cube format using ASE """
        potential, atoms = ase.io.cube.read_cube_data(filename)
        vector_a = np.linalg.norm(atoms.cell[0])
        vector_b = np.linalg.norm(atoms.cell[1])
        vector_c = np.linalg.norm(atoms.cell[2])
        NGX = len(potential)
        NGY = len(potential[0])
        NGZ = len(potential[0][0])
        resolution_x = vector_a/NGX
        resolution_y = vector_b/NGY
        resolution_z = vector_c/NGZ
        coords = atoms.get_positions()
        atom_types = atoms.get_chemical_symbols()

        with open(filename, 'r') as cube:
            lines = cube.readlines()
        origin = [float(i) for i in lines[2].split()[1:-1]]

        return potential, NGX, NGY, NGZ, atoms, [vector_a, vector_b, vector_c], coords, atom_types, origin, [resolution_x, resolution_y, resolution_z]

    def get_CenterOfMass(self):
        """ Returns the NG* values of the Center of Mass """

        atomic_mass = {'H':1.01, 'He':4.00, 'Li':6.94, 'Be':9.01, 'B':10.81, 'C':12.01,
                    'N':14.01, 'O':16.00, 'F':19.00, 'Ne':20.18, 'Na':22.99, 'Mg':24.31,
                    'Al':26.98, 'Si':28.09, 'P':30.97, 'S':32.07, 'Cl':35.45, 'Ar':39.95,
                    'K':39.10, 'Ca':40.08, 'Sc':44.96, 'Ti':47.87, 'V':50.94, 'Cr':52.00,
                    'Mn':54.94, 'Fe':55.85, 'Co':58.93, 'Ni':58.69, 'Cu':63.55, 'Zn':65.39,
                    'Ga':69.72, 'Ge':72.61, 'As':74.92, 'Se':78.96, 'Br':79.90, 'Kr':83.80,
                    'Rb':85.47, 'Sr':87.62, 'Y':88.91, 'Zr':91.22, 'Nb':92.91, 'Mo':95.94,
                    'Tc':98.00, 'Ru':101.07, 'Rh':102.91, 'Pd':106.42, 'Ag':107.87,
                    'Cd':112.41, 'In':114.82, 'Sn':118.71, 'Sb':121.76, 'Te':127.60,
                    'I':126.90, 'Xe':131.29, 'Cs':132.91, 'Ba':137.33, 'La':138.91,
                    'Ce':140.12, 'Pr':140.91, 'Nd':144.24, 'Pm':145.00, 'Sm':150.36,
                    'Eu':151.96, 'Gd':157.25, 'Tb':158.93, 'Dy':162.50, 'Ho':164.93,
                    'Er':167.26, 'Tm':168.93, 'Yb':173.04, 'Lu':174.97, 'Hf':178.49,
                    'Ta':180.95, 'W':183.84, 'Re':186.21, 'Os':190.23, 'Ir':192.22,
                    'Pt':195.08, 'Au':196.97, 'Hg':200.59, 'Tl':204.38, 'Pb':207.2,
                    'Bi':208.98, 'Po':209.00, 'At':210.00, 'Rn':222.00, 'Fr':223.00,
                    'Ra':226.00, 'Ac':227.00, 'Th':232.04, 'Pa':231.04, 'U':238.03,
                    'Np':237.00, 'Pu':244.00, 'Am':243.00, 'Cm':247.00, 'Bk':247.00,
                    'Cf':251.00, 'Es':252.00, 'Fm':257.00, 'Md':258.00, 'No':259.00,
                    'Lr':262.00, 'Rf':261.00, 'Db':262.00, 'Sg':266.00, 'Bh':264.00,
                    'Hs':269.00, 'Mt':268.00}
        total_mass = 0
        comx = 0
        comy = 0
        comz = 0

        for index, row in self.coords_and_atoms.iterrows():
            atom_mass = atomic_mass[str(row['atom'])]
            comx += atom_mass * row['x']
            comy += atom_mass * row['y']
            comz += atom_mass * row['z']
            total_mass += atom_mass
        comx = comx/total_mass
        comy = comy/total_mass
        comz = comz/total_mass
        com = pd.DataFrame(data=[[comx, comy, comz]])
        com.columns = ['x', 'y', 'z']
        return com

    def get_efield(self):
        """ Derives the electric field from the electrostatic potential """
        n = self.n
        ng = self.ngs
        vecs = self.vecs
        vol_per_cube = (vecs[0]/ng[0]) * (vecs[1]/ng[1]) * (vecs[2]/ng[2])

        pgrid = self.pot_grid
        res = [vecs[0]/ng[0], vecs[1]/ng[1], vecs[2]/ng[2]]

        grad_x, grad_y, grad_z = np.gradient(pgrid[:,:,:], res[0], res[1], res[2])

        xy = np.multiply(grad_x, grad_y)
        grad_mag = np.multiply(xy, grad_z)
        # grad_mag = mf.grad_magnitude(grad_x, grad_y, grad_z)
        print(grad_mag.shape)
        self.efield = grad_mag

    def longest_vector(self):
      """ determines the max distance from the center of mass to the surface """
      vac = self.sijk
      vecs = self.vecs
      ngs = self.ngs
      # dist_matrix = np.zeros(shape=(len(vac), 7))
      # row = 0
      distances = []

      for p1 in tqdm(vac):
        for p2 in vac:
          distances.append(gm.distance(p1[0],p2[0]))
      lv = max(distances)
      lvx = (vecs[0]/ngs[0]) * lv * 0.529
      lvy = (vecs[1]/ngs[1]) * lv * 0.529
      lvz = (vecs[2]/ngs[2]) * lv * 0.529

      self.l_vector = np.average([lvx, lvy, lvz])

    def sample_density(self, method):
      """ Samples the electron density """
      dens = self.pot_grid
      ngs = self.ngs
      cv = method

      vacuum = []
      non_vacuum = []
      surf_ijk = []
      surface = []
      vac_ijk = []

      v = 0
      n = 0
      get_first_rise = 0

      for i in tqdm(range(ngs[0]-1)):
        for j in range(ngs[1]-1):
          for k in range(ngs[2]-1):
            if dens[i,j,k] <= method:
              vacuum.append(dens[i,j,k])
              vac_ijk.append([[i,j,k]])
              v += 1
              n = 0
            else:
              non_vacuum.append(dens[i,j,k])
              n += 1
              v = 0
            if (n+v) == 1:
              surface.append(dens[i,j,k])
              surf_ijk.append([[i,j,k]])
      vecs = self.vecs
      vol_per_cube = (vecs[0]/ngs[0]) * (vecs[1]/ngs[1]) * (vecs[2]/ngs[2])
      volume = vol_per_cube * len(non_vacuum) * np.power(0.529, 3)

      self.vol = volume
      #self.vac = vacuum
      #self.vijk = vac_ijk
      #self.surface = surface
      #self.sijk = surf_ijk

    def sample_test(self):
        efield = self.efield
        ngs = self.ngs
        cv = 1e-5
        vacuum = []
        non_vacuum = []
        surf_ijk = []
        surface = []

        v = 0
        n = 0
        get_first_rise = 0
        for i in tqdm(range(ngs[0]-1)):
          for j in range(ngs[1]-1):
            for k in range(ngs[2]-1):
              if np.absolute(efield[i,j,k] - efield[i+1, j+1, k+1]) <= cv:
                vacuum.append(efield[i,j,k])
                v += 1
                n = 0
              else:
                non_vacuum.append(efield[i,j,k])
                n += 1
                v = 0
              if (n+v) == 2:
                if get_first_rise == 0:
                  surface.append(efield[i,j,k])
                  surf_ijk.append([[i,j,k]])
                  needed_mag = efield[i,j,k]
                  get_first_rise += 1
                elif np.absolute(efield[i,j,k] - needed_mag) <= 1e-10:
                  surface.append(efield[i,j,k])
                  surf_ijk.append([[i,j,k]])
                  get_first_rise = 0

        vecs = self.vecs
        vol_per_cube = (vecs[0]/ngs[0]) * (vecs[1]/ngs[1]) * (vecs[2]/ngs[2])
        volume = vol_per_cube * len(non_vacuum) * np.power(0.529, 3)

        self.vol = volume
        self.nvac = non_vacuum
        self.surf = surface
        self.sijk = surf_ijk
        self.vac = vacuum

    def sample_efield(self):
        """ samples the electric field and bins the vac, nvac, and surface cubes """
        efield = self.efield
        ngs = self.ngs
        cv = 1e-5
        vacuum = []
        non_vacuum = []
        surface = []
        vac_ijk = []
        nvac_ijk = pd.DataFrame()
        surf_ijk = []

        v = 0 
        n = 0
        vc = 0
        nc = 0
        get_first_rise = 0
        for i in tqdm(range(ngs[0] - 1)):
            for j in range(ngs[1] - 1):
                count = 1
                for k in range(ngs[2] - 1):
                    if np.absolute(efield[i,j,k] - efield[i + 1, j + 1, k + 1]) <= cv:
                        vacuum.append(efield[i,j,k])
                        v += 1
                        n = 0
                    else:
                        non_vacuum.append(efield[i,j,k])
                        n += 1
                        v = 0
                    if n/count != 1.0 and v/count != 1.0:
                        if get_first_rise == 0:
                            surface.append(efield[i,j,k])
                            surf_ijk.append([[i, j, k]])
                            needed_mag = efield[i,j,k]
                            n = 0
                            v = 0
                            count = 0
                            get_first_rise += 1
                        elif np.absolute(efield[i,j,k] - needed_mag) <= 1e-10:
                            surface.append(efield[i,j,k])
                            surf_ijk.append([[i,j,k]])
                            n = 0
                            v = 0
                            count = 0
                    count += 1
                get_first_rise = 0

        vecs = self.vecs
        vol_per_cube = (vecs[0]/ngs[0]) * (vecs[1]/ngs[1]) * (vecs[2]/ngs[2])
        ubound_vol = vol_per_cube * len(non_vacuum) # * np.power(0.529,3)

        self.vol = ubound_vol
        #self.nvac = non_vacuum
        #self.surf = surface
        self.sijk = surf_ijk
        #self.vac = vacuum

    def get_radii(self):
        """ calculates the min and max radius (between COM and surface"""
        def get_com():
            atomic_mass = {'H':1.01, 'He':4.00, 'Li':6.94, 'Be':9.01, 'B':10.81, 'C':12.01,
                        'N':14.01, 'O':16.00, 'F':19.00, 'Ne':20.18, 'Na':22.99, 'Mg':24.31,
                        'Al':26.98, 'Si':28.09, 'P':30.97, 'S':32.07, 'Cl':35.45, 'Ar':39.95,
                        'K':39.10, 'Ca':40.08, 'Sc':44.96, 'Ti':47.87, 'V':50.94, 'Cr':52.00,
                        'Mn':54.94, 'Fe':55.85, 'Co':58.93, 'Ni':58.69, 'Cu':63.55, 'Zn':65.39,
                        'Ga':69.72, 'Ge':72.61, 'As':74.92, 'Se':78.96, 'Br':79.90, 'Kr':83.80,
                        'Rb':85.47, 'Sr':87.62, 'Y':88.91, 'Zr':91.22, 'Nb':92.91, 'Mo':95.94,
                        'Tc':98.00, 'Ru':101.07, 'Rh':102.91, 'Pd':106.42, 'Ag':107.87,
                        'Cd':112.41, 'In':114.82, 'Sn':118.71, 'Sb':121.76, 'Te':127.60,
                        'I':126.90, 'Xe':131.29, 'Cs':132.91, 'Ba':137.33, 'La':138.91,
                        'Ce':140.12, 'Pr':140.91, 'Nd':144.24, 'Pm':145.00, 'Sm':150.36,
                        'Eu':151.96, 'Gd':157.25, 'Tb':158.93, 'Dy':162.50, 'Ho':164.93,
                        'Er':167.26, 'Tm':168.93, 'Yb':173.04, 'Lu':174.97, 'Hf':178.49,
                        'Ta':180.95, 'W':183.84, 'Re':186.21, 'Os':190.23, 'Ir':192.22,
                        'Pt':195.08, 'Au':196.97, 'Hg':200.59, 'Tl':204.38, 'Pb':207.2,
                        'Bi':208.98, 'Po':209.00, 'At':210.00, 'Rn':222.00, 'Fr':223.00,
                        'Ra':226.00, 'Ac':227.00, 'Th':232.04, 'Pa':231.04, 'U':238.03,
                        'Np':237.00, 'Pu':244.00, 'Am':243.00, 'Cm':247.00, 'Bk':247.00,
                        'Cf':251.00, 'Es':252.00, 'Fm':257.00, 'Md':258.00, 'No':259.00,
                        'Lr':262.00, 'Rf':261.00, 'Db':262.00, 'Sg':266.00, 'Bh':264.00,
                        'Hs':269.00, 'Mt':268.00}
            total_mass = 0
            comx = 0
            comy = 0
            comz = 0

            for index, row in self.coords_and_atoms.iterrows():
                atom_mass = atomic_mass[str(row['atom'])]
                comx += atom_mass * row['x']
                comy += atom_mass * row['y']
                comz += atom_mass * row['z']
                total_mass += atom_mass
            comx = comx/total_mass
            comy = comy/total_mass
            comz = comz/total_mass
            com = pd.DataFrame(data=[[comx, comy, comz]])
            com.columns = ['x', 'y', 'z']
            return com
        vecs = self.vecs
        ngs = self.ngs
        com = get_com()
        print(com)
        com_cube = [int(ngs[0]/2), int(ngs[1]/2), int(ngs[2]/2)]
        print(com_cube)
        sijk = self.sijk
        sijk2 = []
        for p1 in sijk:
            if com_cube == p1[0]:
                pass
            elif p1[0][0] == 0 or p1[0][1] == 0 or p1[0][2] == 0:
                pass
            else:
                sijk2.append(p1)
        arr = np.zeros(shape=(len(sijk2), 4))
        row = 0
        for p1 in sijk2:
            if p1[0][0] == 0 or p1[0][1] == 0 or p1[0][2] == 0:
                print(p1)
            else:
                arr[row, 0] = p1[0][0]
                arr[row, 1] = p1[0][1]
                arr[row, 2] = p1[0][2]
                arr[row, 3] = gm.distance(p1[0], com_cube)
                row += 1

        try:
            abs_min_length = np.min(arr[:,3])
            abs_xminlen = (vecs[0]/ngs[0]) * abs_min_length * 0.529
            abs_yminlen = (vecs[1]/ngs[1]) * abs_min_length * 0.529
            abs_zminlen = (vecs[2]/ngs[2]) * abs_min_length * 0.529

            abs_max_length = np.max(arr[:,3])
            abs_xmaxlen = (vecs[0]/ngs[0]) * abs_max_length * 0.529
            abs_ymaxlen = (vecs[1]/ngs[1]) * abs_max_length * 0.529
            abs_zmaxlen = (vecs[2]/ngs[2]) * abs_max_length * 0.529

            self.min_rad = np.average([abs_xminlen, abs_yminlen, abs_zminlen])
            self.max_rad = np.average([abs_xmaxlen, abs_ymaxlen, abs_zmaxlen])
        except ValueError:
            self.min_rad = 0
            self.max_rad = 0
        # min_rad = np.average([abs_xminlen, abs_yminlen, abs_zminlen])
        # max_rad = np.average([abs_xmaxlen, abs_ymaxlen, abs_zmaxlen])

        # return min_rad, max_rad




from . import math_functions as mf 
import pandas as pd
import numpy as np
import pickle
import math
import os

def gen_speciesANDatoms_line(ncoords):
        species = []
        
        for index, row in ncoords.iterrows():
            species.append(row['atom'])
        atoms = [len(list(group)) for key, group in groupby(species)]
        fspecies = list(set(species))
        return fspecies, atoms

def combine_coordsANDatoms(coords, species, num_atoms=None):
    # Pairs each coordinate with an atom type
	if num_atoms != None:
		species_num = []
		for index, atom in enumerate(species):
			# print(atom, num_atoms[index])
			for i in range(num_atoms[index]):
				species_num.append(atom)
		coordsDF = pd.DataFrame(data=coords, columns=['x','y','z'])
		# print(coordsDF)
		atomsDF = pd.DataFrame(data=species_num, columns=['atom'])
		totalDF = pd.concat([coordsDF, atomsDF], axis=1)
	else:
		coordsDF = pd.DataFrame(data=coords, columns=['x', 'y', 'z'])
		atomsDF = pd.DataFrame(data=species, columns=['atom'])
		totalDF = pd.concat([coordsDF, atomsDF], axis=1)
	return totalDF

def graph3D_DataFrame(b, h=None):
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    ax.scatter(b['x'].astype('float'), b['y'].astype('float'), b['z'].astype('float'))
    try:
        ax.scatter(h['x'].astype('float'), h['y'].astype('float'), h['z'].astype('float'), c='red')
    except Exception:
        pass
    plt.show()

def distance(point1, point2, data_type=None):
	if data_type == 'dataframe':
		return math.sqrt((point1['x']-point2['x'])**2 + (point1['y']-point2['y'])**2 + (point1['z']-point2['z'])**2)
	else:
		return math.sqrt((point1[0]-point2[0])**2 + (point1[1]-point2[1])**2 + (point1[2]-point2[2])**2)
